#ifndef _Symbol_hh_
#define _Symbol_hh_

#include <string>
#include <vector>

#include "common.hh"

class Token;

class TagField
{
public:
  StringSet * tags;
  string rewriter;
  
  virtual string first() const;
  virtual bool contains(string) const;
  virtual string toString() const;
  
  TagField(StringSet * set=0, string rw=""):
    tags(set), rewriter(rw)
    {
      if(tags==0) tags = new StringSet();
    }
  
  TagField(string tag, string rw=""):
    rewriter(rw)
    {
      tags = new StringSet();
      tags->insert(tag);
    }
  
  virtual bool isEmpty() const
    {
      return tags->size() == 0 && rewriter =="" ;
    }
  
  virtual ~TagField()
    {}
};

class FormAndFunc
{
public:
  TagField * func;
  TagField * form;
  
  FormAndFunc(TagField* fu=0, TagField* fo=0):
    func(fu), form(fo)
    {
      if(func==0) func = new TagField();
      if(form==0) form = new TagField();
    }
  
  ~FormAndFunc(){}
};

class Symbol
{
 public:
  static Symbol * const SKIP_SYMBOL;
  static Symbol * const EPSILON_SYMBOL;
  
  TagField* funcs;
  TagField* forms;
  TagField* sfuncs;
  TagField* sforms;
  TagField* wordforms;
  TagField* baseforms;
  
  virtual string toString() const;
  virtual Token* rewrite(Token*) const;
  
  Symbol(TagField* fu  = 0 ,
	 TagField* fo  = 0 ,
	 TagField* sfu = 0 ,
	 TagField* sfo = 0 ,
	 TagField* wor = 0 ,
	 TagField* bas = 0 ) :
    funcs(fu), forms(fo),
    sfuncs(sfu), sforms(sfo),
    wordforms(wor), baseforms(bas)
    {
      if(funcs==0) funcs = new TagField();
      if(forms==0) forms = new TagField();
      if(sfuncs==0) sfuncs = new TagField();
      if(sforms==0) sforms = new TagField();
      if(wordforms==0) wordforms = new TagField();
      if(baseforms==0) baseforms = new TagField();
    }
  
  virtual bool isEmpty() const;
  virtual bool isPunctuation() const;
  
  virtual ~Symbol()
    {}
};

#endif
